<?php
/*
Plugin Name: PageAICategoryCreator
Description: Auto-generate and assign diverse, unique categories to pages via DeepSeek API and list posts that share them.
Version: 3.3
Author: VoiceLessQ
Author URI:
Text Domain: page-ai-categories
*/

// PROTECTED CODE - DO NOT MODIFY
// Copyright (C) 2025 VoiceLessQ
// This code is protected by copyright law and is not intended for redistribution.



// Define plugin version constant
if (!defined('PLUGIN_VERSION')) {
    define('PLUGIN_VERSION', '3.3');
}

if (! defined('ABSPATH')) {
    exit;
}

// DeepSeek API settings
if (! defined('DEEPSEEK_API_ENDPOINT')) {
    define('DEEPSEEK_API_ENDPOINT', 'https://api.deepseek.com/v1/chat/completions');
}
if (! defined('DEEPSEEK_MODEL')) {
    define('DEEPSEEK_MODEL', 'deepseek-chat');
}

// Get API key from options
function page_ai_get_api_key() {
    $options = page_ai_get_options();
    return isset($options['api_key']) ? $options['api_key'] : '';
}

// 1. Enable categories on pages
add_action('init', 'page_ai_enable_categories_for_pages', 10);

/**
 * Enable categories for pages
 */
function page_ai_enable_categories_for_pages() {
    // Register category taxonomy for pages
    register_taxonomy_for_object_type('category', 'page');
    add_post_type_support('page', ['category']);

    // Log the status for debugging
    if (WP_DEBUG) {
        $is_registered = is_object_in_taxonomy('page', 'category');
        error_log('Page AI Categories: Category taxonomy ' . ($is_registered ? 'IS' : 'IS NOT') . ' registered for pages');
    }
}

// 2. Auto-generate on first publish
add_action('transition_post_status', function($new_status, $old_status, $post) {
    if ($post->post_type !== 'page') {
        return;
    }
    if ($old_status === $new_status || $new_status !== 'publish') {
        return;
    }
    // generate only if no categories yet
    $cats = wp_get_post_terms($post->ID, 'category', ['fields' => 'ids']);
    if (empty($cats)) {
        // Generate categories for this page
        page_ai_generate_categories_for_page($post->ID, $post->post_title);

        // Check for and fix duplicate categories if the option is enabled
        $options = page_ai_get_options();
        if ((bool)$options['unique_categories']) {
            // Get all pages to check for duplicates
            $all_pages = get_posts([
                'post_type'      => 'page',
                'post_status'    => 'publish',
                'posts_per_page' => -1,
                'fields'         => 'ids',
            ]);

            // Pass the current page ID as priority to ensure its categories are preserved
            page_ai_fix_duplicate_categories($all_pages, $post->ID);
        }
    }
}, 10, 3);

// 3. Re-check on save if uncategorized
add_action('save_post_page', function($post_id, $post, $update) {
    if (wp_is_post_autosave($post_id) || wp_is_post_revision($post_id)) {
        return;
    }
    if ($post->post_type !== 'page') {
        return;
    }
    // only if no categories assigned
    $cats = wp_get_post_terms($post_id, 'category', ['fields' => 'ids']);
    if (empty($cats)) {
        // Generate categories for this page
        page_ai_generate_categories_for_page($post_id, $post->post_title);

        // Check for and fix duplicate categories if the option is enabled
        $options = page_ai_get_options();
        if ((bool)$options['unique_categories']) {
            // Get all pages to check for duplicates
            $all_pages = get_posts([
                'post_type'      => 'page',
                'post_status'    => 'publish',
                'posts_per_page' => -1,
                'fields'         => 'ids',
            ]);

            // Pass the current page ID as priority to ensure its categories are preserved
            page_ai_fix_duplicate_categories($all_pages, $post_id);
        }
    } else {
        // If categories exist, just check for duplicates within this page
        // This ensures we keep the existing categories but remove any duplicates
        $unique_cats = page_ai_remove_duplicate_category_names($cats);
        if (count($unique_cats) !== count($cats)) {
            // Only update if we actually removed duplicates
            wp_set_post_terms($post_id, $unique_cats, 'category', false);
        }
    }
}, 20, 3);

// 4. Daily cron to catch misses
register_activation_hook(__FILE__, 'page_ai_categories_activate');
register_deactivation_hook(__FILE__, function() {
    wp_clear_scheduled_hook('page_ai_daily_recheck');
});

/**
 * Plugin activation: flush rules and auto-generate categories for all existing pages
 */
function page_ai_categories_activate() {
    // Ensure rewrite rules are fresh
    flush_rewrite_rules();

    // Auto-assign AI categories for all existing published pages
    $pages = get_posts([
        'post_type'      => 'page',
        'post_status'    => 'publish',
        'numberposts'    => -1,
        'fields'         => 'ids',
    ]);

    // Get plugin options
    $options = page_ai_get_options();
    $unique_categories = (bool)$options['unique_categories'];
    $exclude_default = (bool)$options['exclude_default_pages'];
    $excluded_pages = isset($options['excluded_pages']) ? $options['excluded_pages'] : [];
    $excluded_menu_items = isset($options['excluded_menu_items']) ? $options['excluded_menu_items'] : [];
    $only_remove_from_excluded = (bool)$options['only_remove_from_excluded'];

    // Get selection mode and included pages
    $selection_mode = isset($options['selection_mode']) ? $options['selection_mode'] : 'exclude';
    $included_pages = isset($options['included_pages']) ? $options['included_pages'] : [];

    // Process each page
    foreach ($pages as $pid) {
        $p = get_post($pid);

        // Determine if this page should be processed based on selection mode
        $should_process = true;

        if ($selection_mode === 'exclude') {
            // In exclude mode, process all pages except excluded ones
            $is_excluded = false;

            // Check if it's a default page
            if ($exclude_default && page_ai_is_default_page($pid)) {
                $is_excluded = true;
            }

            // Check if it's in the excluded pages list
            if (in_array($pid, $excluded_pages)) {
                $is_excluded = true;
            }

            // Check if it's in the excluded menu items list
            if (in_array($pid, $excluded_menu_items)) {
                $is_excluded = true;
            }

            // If the page is excluded, remove categories and continue
            if ($is_excluded) {
                // Remove any existing categories from this page
                wp_set_post_terms($pid, [], 'category', false);
                continue;
            }
        } else {
            // In include mode, only process specifically included pages
            $is_included = false;

            // Check if it's in the included pages list
            if (in_array($pid, $included_pages)) {
                $is_included = true;
            }

            // Check if it's in the included menu items list (which uses the same field as excluded_menu_items)
            if (in_array($pid, $excluded_menu_items)) {
                $is_included = true;
            }

            // If the page is not included, don't process it
            if (!$is_included) {
                // If we're only removing categories from excluded pages, preserve existing categories
                if (!$only_remove_from_excluded) {
                    // Remove any existing categories from this page
                    wp_set_post_terms($pid, [], 'category', false);
                }
                continue;
            }
        }

        // If we're only removing categories from excluded pages and this page isn't excluded,
        // preserve its existing categories
        if ($only_remove_from_excluded) {
            $cats = wp_get_post_terms($pid, 'category', ['fields' => 'ids']);
            if (!empty($cats)) {
                continue;
            }
        }

        // Generate categories for pages without them
        $cats = wp_get_post_terms($pid, 'category', ['fields'=>'ids']);
        if (empty($cats)) {
            page_ai_generate_categories_for_page($pid, $p->post_title);
        }
    }

    // Check for and fix duplicate categories if the option is enabled
    if ($unique_categories) {
        // Get pages with categories to prioritize them
        $pages_with_categories = [];
        foreach ($pages as $pid) {
            $cats = wp_get_post_terms($pid, 'category', ['fields' => 'ids']);
            if (!empty($cats)) {
                $pages_with_categories[] = $pid;
            }
        }

        // Process each page with categories as a priority to preserve its categories
        foreach ($pages_with_categories as $priority_page) {
            // Get remaining pages excluding the current priority page
            $other_pages = array_diff($pages, [$priority_page]);

            // Fix duplicates with this page as priority
            page_ai_fix_duplicate_categories(array_merge([$priority_page], $other_pages), $priority_page);
        }

        // Process any remaining pages without categories
        $pages_without_categories = array_diff($pages, $pages_with_categories);
        if (!empty($pages_without_categories)) {
            page_ai_fix_duplicate_categories($pages_without_categories);
        }
    }

    // Schedule daily recheck
    if (! wp_next_scheduled('page_ai_daily_recheck')) {
        wp_schedule_event(time(), 'daily', 'page_ai_daily_recheck');
    }
}

// Daily recheck function
function page_ai_daily_recheck_callback() {
    $options = page_ai_get_options();
    $force_reassign = (bool)$options['force_reassign'];
    $unique_categories = (bool)$options['unique_categories'];

    // First, let's check all pages regardless of category status
    $all_pages = get_posts([
        'post_type'      => 'page',
        'post_status'    => 'publish',
        'posts_per_page' => -1,
        'fields'         => 'ids',
    ]);

    // If unique categories is enabled, check for and fix duplicate categories
    $fixed_duplicates = 0;
    if ($unique_categories) {
        // Get pages with categories to prioritize them
        $pages_with_categories = [];
        foreach ($all_pages as $pid) {
            $cats = wp_get_post_terms($pid, 'category', ['fields' => 'ids']);
            if (!empty($cats)) {
                $pages_with_categories[] = $pid;
            }
        }

        // Process each page with categories as a priority to preserve its categories
        foreach ($pages_with_categories as $priority_page) {
            // Get remaining pages excluding the current priority page
            $other_pages = array_diff($all_pages, [$priority_page]);

            // Fix duplicates with this page as priority
            $fixed_duplicates += page_ai_fix_duplicate_categories(array_merge([$priority_page], $other_pages), $priority_page);
        }

        // Process any remaining pages without categories
        $pages_without_categories = array_diff($all_pages, $pages_with_categories);
        if (!empty($pages_without_categories)) {
            $fixed_duplicates += page_ai_fix_duplicate_categories($pages_without_categories);
        }
    }

    // Get pages to process
    if ($force_reassign) {
        // If force reassign is enabled, process all pages
        $pages = $all_pages;
    } else {
        // Otherwise, only process pages without categories
        $pages = get_posts([
            'post_type'      => 'page',
            'post_status'    => 'publish',
            'posts_per_page' => -1,
            'fields'         => 'ids',
            'tax_query'      => [[
                'taxonomy' => 'category',
                'operator' => 'NOT EXISTS',
            ]],
        ]);
    }

    $count = 0;
    $debug_info = [];

    // Store debug info about all pages
    $debug_info['total_pages'] = count($all_pages);
    $debug_info['pages_without_categories'] = count($pages);
    $debug_info['force_reassign'] = $force_reassign;
    $debug_info['unique_categories'] = $unique_categories;
    $debug_info['exclude_default_pages'] = (bool)$options['exclude_default_pages'];
    $debug_info['fixed_duplicates'] = $fixed_duplicates;
    $debug_info['page_details'] = [];

    // Count default pages
    $default_pages = [];
    foreach ($all_pages as $pid) {
        if (page_ai_is_default_page($pid)) {
            $default_pages[] = $pid;
        }
    }
    $debug_info['default_pages_count'] = count($default_pages);

    // Get menu information
    $menu_pages = page_ai_get_menu_pages();
    $debug_info['menus'] = wp_get_nav_menus();
    $debug_info['menu_pages'] = $menu_pages;
    $debug_info['menu_pages_count'] = count($menu_pages);

    // Get info about each page
    foreach ($all_pages as $pid) {
        $p = get_post($pid);
        $cats = wp_get_post_terms($pid, 'category');
        $cat_ids = wp_list_pluck($cats, 'term_id');
        $cat_names = wp_list_pluck($cats, 'name');
        $is_default = page_ai_is_default_page($pid);

        $debug_info['page_details'][] = [
            'id' => $pid,
            'title' => $p->post_title,
            'is_default_page' => $is_default,
            'has_categories' => !empty($cats),
            'category_count' => count($cats),
            'category_ids' => $cat_ids,
            'category_names' => $cat_names,
        ];
    }

    // Process pages without categories
    foreach ($pages as $pid) {
        $p = get_post($pid);

        // Skip default pages if the option is enabled
        if ($options['exclude_default_pages'] && page_ai_is_default_page($pid)) {
            // Remove any existing categories from this page
            wp_set_post_terms($pid, [], 'category', false);
            continue;
        }

        $result = page_ai_generate_categories_for_page($pid, $p->post_title);
        if ($result) {
            $count++;
        }

        // Log results if enabled
        if ($options['error_logging']) {
            error_log(sprintf(
                'Page AI Categories: %s categories for page #%d "%s"',
                $result ? 'Generated' : 'Failed to generate',
                $pid,
                $p->post_title
            ));
        }
    }

    // Store last run stats with debug info
    update_option('page_ai_last_run', [
        'time' => current_time('mysql'),
        'count' => $count,
        'total' => count($pages),
        'debug_info' => $debug_info
    ]);
}

// Register the daily recheck callback
add_action('page_ai_daily_recheck', 'page_ai_daily_recheck_callback');

// Schedule on activation
register_activation_hook(__FILE__, function() {
    if (! wp_next_scheduled('page_ai_daily_recheck')) {
        wp_schedule_event(time(), 'daily', 'page_ai_daily_recheck');
    }
});

// Clear schedule on deactivation
register_deactivation_hook(__FILE__, function() {
    wp_clear_scheduled_hook('page_ai_daily_recheck');
});

// Add admin menu and settings page
add_action('admin_menu', 'page_ai_add_admin_menu');
add_action('admin_init', 'page_ai_settings_init');

// Enqueue admin scripts and styles
add_action('admin_enqueue_scripts', 'page_ai_admin_enqueue_scripts');

/**
 * Enqueue admin scripts and styles
 */
function page_ai_admin_enqueue_scripts($hook) {
    // Only enqueue on our plugin page
    if ('toplevel_page_page_ai_categories' !== $hook) {
        return;
    }

    // Enqueue jQuery UI for sortable
    wp_enqueue_script('jquery-ui-sortable');

    // Enqueue WordPress color picker
    wp_enqueue_style('wp-color-picker');
    wp_enqueue_script('wp-color-picker');

    // Enqueue our custom scripts and styles
    wp_enqueue_style('page-ai-admin-css', plugin_dir_url(__FILE__) . 'assets/css/admin.css', [], '1.0.0');
    wp_enqueue_script('page-ai-admin-js', plugin_dir_url(__FILE__) . 'assets/js/admin.js', ['jquery', 'jquery-ui-sortable', 'wp-color-picker'], '1.0.0', true);
}

// Clear cache when categories are updated
add_action('edited_category', 'page_ai_clear_category_cache');
add_action('create_category', 'page_ai_clear_category_cache');
add_action('delete_category', 'page_ai_clear_category_cache');
add_action('set_object_terms', 'page_ai_clear_page_category_cache', 10, 4);

/**
 * Clear category-related caches when categories are updated
 */
function page_ai_clear_category_cache() {
    global $wpdb;

    // Delete all transients related to category usage
    $wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE '%_transient_page_ai_cat_%'");
}

/**
 * Clear page-specific category caches when terms are set for a page
 */
function page_ai_clear_page_category_cache($object_id, $terms, $tt_ids, $taxonomy) {
    // Only process category taxonomy for pages
    if ($taxonomy !== 'category') {
        return;
    }

    $post_type = get_post_type($object_id);
    if ($post_type !== 'page') {
        return;
    }

    global $wpdb;

    // Delete transients related to this page
    $wpdb->query($wpdb->prepare(
        "DELETE FROM $wpdb->options WHERE option_name LIKE '%_transient_page_ai_cat_%%_%d%%'",
        $object_id
    ));
}

/**
 * Add menu item to WordPress admin
 */
function page_ai_add_admin_menu() {
    add_menu_page(
        'Page AI Categories',
        'Page Categories',
        'manage_options',
        'page_ai_categories',
        'page_ai_settings_page',
        'dashicons-category',
        30
    );
}

/**
 * Register settings
 */
function page_ai_settings_init() {
    register_setting('page_ai_settings', 'page_ai_categories_options', 'page_ai_validate_options');

    // API Settings Section
    add_settings_section(
        'page_ai_api_settings_section',
        'API Settings',
        'page_ai_api_settings_section_callback',
        'page_ai_settings'
    );

    add_settings_field(
        'api_key',
        'DeepSeek API Key',
        'page_ai_api_key_render',
        'page_ai_settings',
        'page_ai_api_settings_section'
    );

    // Category Generation Settings Section
    add_settings_section(
        'page_ai_settings_section',
        'Category Generation Settings',
        'page_ai_settings_section_callback',
        'page_ai_settings'
    );

    add_settings_field(
        'categories_per_page',
        'Max Categories Per Page',
        'page_ai_categories_per_page_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'use_existing_categories',
        'Use Existing Categories',
        'page_ai_use_existing_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'create_new_categories',
        'Create New Categories',
        'page_ai_create_new_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'error_logging',
        'Enable Error Logging',
        'page_ai_error_logging_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'force_reassign',
        'Force Reassign Categories',
        'page_ai_force_reassign_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'unique_categories',
        'Unique Categories Per Page',
        'page_ai_unique_categories_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'exclude_default_pages',
        'Exclude Default WordPress Pages',
        'page_ai_exclude_default_pages_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );

    add_settings_field(
        'strict_unique_categories',
        'Strict Unique Categories',
        'page_ai_strict_unique_categories_render',
        'page_ai_settings',
        'page_ai_settings_section'
    );
}

/**
 * API Settings section description
 */
function page_ai_api_settings_section_callback() {
    echo '<p>Configure your DeepSeek API settings. You need a valid API key to generate categories.</p>';
}

/**
 * Render API key field
 */
function page_ai_api_key_render() {
    $options = page_ai_get_options();
    $api_key = isset($options['api_key']) ? $options['api_key'] : '';
    ?>
    <input type='password' name='page_ai_categories_options[api_key]' value='<?php echo esc_attr($api_key); ?>' class="regular-text" autocomplete="off">
    <p class="description">Enter your DeepSeek API key. <a href="https://www.deepseek.com/" target="_blank">Get an API key</a></p>
    <?php
}

/**
 * Settings section description
 */
function page_ai_settings_section_callback() {
    echo '<p>Configure how categories are generated and assigned to pages.</p>';
}

/**
 * Render categories per page field
 */
function page_ai_categories_per_page_render() {
    $options = page_ai_get_options();
    ?>
    <input type='number' name='page_ai_categories_options[categories_per_page]' min='1' value='<?php echo $options["categories_per_page"]; ?>'>
    <p class="description">Maximum number of categories to assign to each page (no upper limit)</p>
    <?php
}

/**
 * Render use existing categories checkbox
 */
function page_ai_use_existing_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[use_existing_categories]' <?php checked($options['use_existing_categories'], 1); ?> value='1'>
    <p class="description">When enabled, the plugin will try to use existing categories</p>
    <?php
}

/**
 * Render create new categories checkbox
 */
function page_ai_create_new_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[create_new_categories]' <?php checked($options['create_new_categories'], 1); ?> value='1'>
    <p class="description">When enabled, the plugin will create new categories as needed</p>
    <?php
}

/**
 * Render error logging checkbox
 */
function page_ai_error_logging_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[error_logging]' <?php checked($options['error_logging'], 1); ?> value='1'>
    <p class="description">Enable logging of category generation results to the error log</p>
    <?php
}

/**
 * Render force reassign checkbox
 */
function page_ai_force_reassign_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[force_reassign]' <?php checked($options['force_reassign'], 1); ?> value='1'>
    <p class="description">Force reassign categories even to pages that already have categories</p>
    <?php
}

/**
 * Render unique categories checkbox
 */
function page_ai_unique_categories_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[unique_categories]' <?php checked($options['unique_categories'], 1); ?> value='1'>
    <p class="description">Ensure each page has unique categories not used by other pages. When duplicates are found, they will be automatically replaced with new AI-generated categories.</p>
    <?php
}

/**
 * Render exclude default pages checkbox
 */
function page_ai_exclude_default_pages_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[exclude_default_pages]' <?php checked($options['exclude_default_pages'], 1); ?> value='1'>
    <p class="description">Exclude WordPress default pages (like Home and Privacy Policy) from category assignment.</p>
    <?php
}

/**
 * Render strict unique categories checkbox
 */
function page_ai_strict_unique_categories_render() {
    $options = page_ai_get_options();
    ?>
    <input type='checkbox' name='page_ai_categories_options[strict_unique_categories]' <?php checked($options['strict_unique_categories'], 1); ?> value='1'>
    <p class="description">Ensure each category is used by only one page. When a category is found on multiple pages, it will be kept on one page and replaced with similar categories on other pages.</p>
    <?php
}

/**
 * Render settings page
 */
function page_ai_settings_page() {
    // Check user capabilities
    if (!current_user_can('manage_options')) {
        return;
    }

    // Add error/update messages
    if (isset($_GET['settings-updated'])) {
        add_settings_error('page_ai_messages', 'page_ai_message', 'Settings Saved', 'updated');
    }

    // Show error/update messages
    settings_errors('page_ai_messages');

    // Get last run info
    $last_run = get_option('page_ai_last_run', ['time' => 'Never', 'count' => 0, 'total' => 0, 'debug_info' => []]);

    // Get settings order or use default
    $options = page_ai_get_options();
    $settings_order = isset($options['settings_order']) ? json_decode($options['settings_order'], true) : [
        'api_key',
        'categories_per_page',
        'use_existing_categories',
        'create_new_categories',
        'unique_categories',
        'strict_unique_categories',
        'exclude_default_pages',
        'force_reassign',
        'error_logging'
    ];

    // Define settings cards
    $settings_cards = [
        'api_key' => [
            'title' => 'DeepSeek API Key',
            'icon' => 'dashicons-admin-network',
            'description' => 'Enter your DeepSeek API key to enable AI-powered category generation.',
            'content' => function() {
                $options = page_ai_get_options();
                $api_key = isset($options['api_key']) ? $options['api_key'] : '';
                ?>
                <div class="page-ai-form-row">
                    <label for="page_ai_api_key">API Key</label>
                    <div class="page-ai-api-key-field">
                        <input type="password" id="page_ai_api_key" name="page_ai_categories_options[api_key]" value="<?php echo esc_attr($api_key); ?>" class="regular-text">
                        <button id="show-api-key" class="button button-secondary">Show</button>
                    </div>
                    <p class="description">Enter your DeepSeek API key. <a href="https://www.deepseek.com/" target="_blank">Get an API key</a></p>
                </div>
                <?php
            }
        ],
        'excluded_shop_categories' => [
            'title' => 'Exclude Shop Categories',
            'icon' => 'dashicons-cart',
            'description' => 'Select shop/e-commerce categories to exclude from automatic assignment.',
            'content' => function() {
                $options = page_ai_get_options();
                $excluded_categories = isset($options['excluded_shop_categories']) ? $options['excluded_shop_categories'] : [];

                // Get all categories
                $categories = get_categories([
                    'hide_empty' => false,
                    'orderby' => 'name',
                    'order' => 'ASC',
                ]);
                ?>
                <div class="page-ai-form-row">
                    <label for="page_ai_excluded_shop_categories">Shop Categories to Exclude</label>
                    <div class="page-ai-category-selector">
                        <?php if (!empty($categories)): ?>
                            <div class="page-ai-category-list">
                                <?php foreach ($categories as $category): ?>
                                    <div class="page-ai-category-item">
                                        <label>
                                            <input type="checkbox"
                                                name="page_ai_categories_options[excluded_shop_categories][]"
                                                value="<?php echo esc_attr($category->term_id); ?>"
                                                <?php checked(in_array($category->term_id, $excluded_categories), true); ?>>
                                            <?php echo esc_html($category->name); ?>
                                            <span class="page-ai-category-count">(<?php echo intval($category->count); ?>)</span>
                                        </label>
                                    </div>
                                <?php endforeach; ?>
                            </div>
                            <p class="description">Select shop or e-commerce categories that should not be used by the automatic category generator. This prevents the plugin from interfering with your shop organization.</p>
                        <?php else: ?>
                            <p>No categories found. Create some categories first.</p>
                        <?php endif; ?>
                    </div>
                </div>
                <?php
            }
        ],
        'selection_mode' => [
            'title' => 'Page Selection Mode',
            'icon' => 'dashicons-admin-settings',
            'description' => 'Choose whether to include or exclude specific pages from automatic category assignment.',
            'content' => function() {
                $options = page_ai_get_options();
                $selection_mode = isset($options['selection_mode']) ? $options['selection_mode'] : 'exclude';
                ?>
                <div class="page-ai-form-row">
                    <label>Selection Mode</label>
                    <div class="page-ai-radio-group">
                        <label class="page-ai-radio-label">
                            <input type="radio" name="page_ai_categories_options[selection_mode]" value="exclude" <?php checked($selection_mode, 'exclude'); ?>>
                            <span>Exclude Mode</span>
                            <p class="description">Auto-categorize all pages except those you select to exclude</p>
                        </label>
                        <label class="page-ai-radio-label">
                            <input type="radio" name="page_ai_categories_options[selection_mode]" value="include" <?php checked($selection_mode, 'include'); ?>>
                            <span>Include Mode</span>
                            <p class="description">Only auto-categorize the specific pages you select to include</p>
                        </label>
                    </div>
                </div>
                <?php
            }
        ],
        'excluded_pages' => [
            'title' => 'Exclude Specific Pages',
            'icon' => 'dashicons-admin-page',
            'description' => 'Select specific pages to exclude from automatic category assignment.',
            'content' => function() {
                $options = page_ai_get_options();
                $excluded_pages = isset($options['excluded_pages']) ? $options['excluded_pages'] : [];
                $selection_mode = isset($options['selection_mode']) ? $options['selection_mode'] : 'exclude';

                // Don't show if in include mode
                if ($selection_mode === 'include') {
                    ?>
                    <div class="page-ai-notice page-ai-notice-info">
                        <p>This setting is not used in "Include Mode". Please use the "Pages to Auto-Categorize" setting instead.</p>
                    </div>
                    <?php
                    return;
                }

                // Get all published pages with hierarchical structure
                $pages = get_pages([
                    'post_status' => 'publish',
                    'sort_column' => 'menu_order, post_title',
                    'sort_order' => 'ASC',
                ]);

                // Organize pages by parent
                $pages_by_parent = [];
                foreach ($pages as $page) {
                    $parent_id = $page->post_parent;
                    if (!isset($pages_by_parent[$parent_id])) {
                        $pages_by_parent[$parent_id] = [];
                    }
                    $pages_by_parent[$parent_id][] = $page;
                }

                // Function to display pages hierarchically
                function display_pages_hierarchically($parent_id, $pages_by_parent, $excluded_pages, $level = 0) {
                    if (!isset($pages_by_parent[$parent_id])) {
                        return;
                    }

                    foreach ($pages_by_parent[$parent_id] as $page) {
                        ?>
                        <div class="page-ai-page-item" style="padding-left: <?php echo $level * 20; ?>px;">
                            <label>
                                <input type="checkbox"
                                    name="page_ai_categories_options[excluded_pages][]"
                                    value="<?php echo esc_attr($page->ID); ?>"
                                    <?php checked(in_array($page->ID, $excluded_pages), true); ?>>
                                <?php echo esc_html($page->post_title); ?>
                                <?php if (page_ai_is_default_page($page->ID)): ?>
                                    <span class="page-ai-page-default">(Default Page)</span>
                                <?php endif; ?>
                            </label>
                        </div>
                        <?php
                        // Display children
                        if (isset($pages_by_parent[$page->ID])) {
                            display_pages_hierarchically($page->ID, $pages_by_parent, $excluded_pages, $level + 1);
                        }
                    }
                }
                ?>
                <div class="page-ai-form-row">
                    <label for="page_ai_excluded_pages">Pages to Exclude</label>
                    <div class="page-ai-page-selector">
                        <div class="page-ai-actions">
                            <button type="button" class="button page-ai-select-all">Select All</button>
                            <button type="button" class="button page-ai-select-none">Select None</button>
                            <button type="button" class="button page-ai-select-default">Select Default Pages</button>
                        </div>
                        <?php if (!empty($pages)): ?>
                            <div class="page-ai-page-list">
                                <?php display_pages_hierarchically(0, $pages_by_parent, $excluded_pages); ?>
                            </div>
                            <p class="description">Select specific pages that should be excluded from automatic category assignment. Categories will be removed from these pages if "Only Remove Categories from Excluded Pages" is enabled.</p>
                        <?php else: ?>
                            <p>No pages found.</p>
                        <?php endif; ?>
                    </div>
                </div>
                <?php
            }
        ],
        'included_pages' => [
            'title' => 'Pages to Auto-Categorize',
            'icon' => 'dashicons-admin-page',
            'description' => 'Select specific pages to include in automatic category assignment.',
            'content' => function() {
                $options = page_ai_get_options();
                $included_pages = isset($options['included_pages']) ? $options['included_pages'] : [];
                $selection_mode = isset($options['selection_mode']) ? $options['selection_mode'] : 'exclude';

                // Don't show if in exclude mode
                if ($selection_mode === 'exclude') {
                    ?>
                    <div class="page-ai-notice page-ai-notice-info">
                        <p>This setting is not used in "Exclude Mode". Please use the "Exclude Specific Pages" setting instead.</p>
                    </div>
                    <?php
                    return;
                }

                // Get all published pages with hierarchical structure
                $pages = get_pages([
                    'post_status' => 'publish',
                    'sort_column' => 'menu_order, post_title',
                    'sort_order' => 'ASC',
                ]);

                // Organize pages by parent
                $pages_by_parent = [];
                foreach ($pages as $page) {
                    $parent_id = $page->post_parent;
                    if (!isset($pages_by_parent[$parent_id])) {
                        $pages_by_parent[$parent_id] = [];
                    }
                    $pages_by_parent[$parent_id][] = $page;
                }

                // Function to display pages hierarchically
                function display_included_pages_hierarchically($parent_id, $pages_by_parent, $included_pages, $level = 0) {
                    if (!isset($pages_by_parent[$parent_id])) {
                        return;
                    }

                    foreach ($pages_by_parent[$parent_id] as $page) {
                        ?>
                        <div class="page-ai-page-item" style="padding-left: <?php echo $level * 20; ?>px;">
                            <label>
                                <input type="checkbox"
                                    name="page_ai_categories_options[included_pages][]"
                                    value="<?php echo esc_attr($page->ID); ?>"
                                    <?php checked(in_array($page->ID, $included_pages), true); ?>>
                                <?php echo esc_html($page->post_title); ?>
                                <?php if (page_ai_is_default_page($page->ID)): ?>
                                    <span class="page-ai-page-default">(Default Page)</span>
                                <?php endif; ?>
                            </label>
                        </div>
                        <?php
                        // Display children
                        if (isset($pages_by_parent[$page->ID])) {
                            display_included_pages_hierarchically($page->ID, $pages_by_parent, $included_pages, $level + 1);
                        }
                    }
                }
                ?>
                <div class="page-ai-form-row">
                    <label for="page_ai_included_pages">Pages to Auto-Categorize</label>
                    <div class="page-ai-page-selector">
                        <div class="page-ai-actions">
                            <button type="button" class="button page-ai-select-all">Select All</button>
                            <button type="button" class="button page-ai-select-none">Select None</button>
                            <button type="button" class="button page-ai-select-non-default">Select Non-Default Pages</button>
                        </div>
                        <?php if (!empty($pages)): ?>
                            <div class="page-ai-page-list">
                                <?php display_included_pages_hierarchically(0, $pages_by_parent, $included_pages); ?>
                            </div>
                            <p class="description">Select specific pages that should be included in automatic category assignment. Only these pages will receive categories.</p>
                        <?php else: ?>
                            <p>No pages found.</p>
                        <?php endif; ?>
                    </div>
                </div>
                <?php
            }
        ],
        'excluded_menu_items' => [
            'title' => 'Menu Items',
            'icon' => 'dashicons-menu',
            'description' => 'Select menu items to exclude or include based on your selection mode.',
            'content' => function() {
                $options = page_ai_get_options();
                $excluded_menu_items = isset($options['excluded_menu_items']) ? $options['excluded_menu_items'] : [];
                $selection_mode = isset($options['selection_mode']) ? $options['selection_mode'] : 'exclude';

                // Get all menus
                $menus = wp_get_nav_menus();

                // Organize menu items by menu and parent
                $menu_items_by_menu = [];
                if (!empty($menus)) {
                    foreach ($menus as $menu) {
                        $items = wp_get_nav_menu_items($menu->term_id);
                        if (!empty($items)) {
                            // Organize by parent
                            $items_by_parent = [];
                            foreach ($items as $item) {
                                if ($item->object !== 'page') {
                                    continue;
                                }

                                $parent_id = $item->menu_item_parent;
                                if (!isset($items_by_parent[$parent_id])) {
                                    $items_by_parent[$parent_id] = [];
                                }
                                $items_by_parent[$parent_id][] = $item;
                            }

                            $menu_items_by_menu[$menu->term_id] = [
                                'name' => $menu->name,
                                'items_by_parent' => $items_by_parent
                            ];
                        }
                    }
                }

                // Function to display menu items hierarchically
                function display_menu_items_hierarchically($parent_id, $items_by_parent, $excluded_menu_items, $level = 0) {
                    if (!isset($items_by_parent[$parent_id])) {
                        return;
                    }

                    foreach ($items_by_parent[$parent_id] as $item) {
                        ?>
                        <div class="page-ai-menu-item" style="padding-left: <?php echo $level * 20; ?>px;">
                            <label>
                                <input type="checkbox"
                                    name="page_ai_categories_options[excluded_menu_items][]"
                                    value="<?php echo esc_attr($item->object_id); ?>"
                                    data-page-id="<?php echo esc_attr($item->object_id); ?>"
                                    <?php checked(in_array($item->object_id, $excluded_menu_items), true); ?>>
                                <?php echo esc_html($item->title); ?>
                                <?php if (page_ai_is_default_page($item->object_id)): ?>
                                    <span class="page-ai-page-default">(Default Page)</span>
                                <?php endif; ?>
                                <span class="page-ai-menu-item-page">(Page: <?php echo get_the_title($item->object_id); ?>)</span>
                            </label>
                        </div>
                        <?php
                        // Display children
                        if (isset($items_by_parent[$item->ID])) {
                            display_menu_items_hierarchically($item->ID, $items_by_parent, $excluded_menu_items, $level + 1);
                        }
                    }
                }
                ?>
                <div class="page-ai-form-row">
                    <label for="page_ai_excluded_menu_items">
                        <?php echo $selection_mode === 'exclude' ? 'Menu Items to Exclude' : 'Menu Items to Include'; ?>
                    </label>
                    <div class="page-ai-menu-selector">
                        <?php if (!empty($menu_items_by_menu)): ?>
                            <div class="page-ai-actions">
                                <button type="button" class="button page-ai-select-all-menu">Select All</button>
                                <button type="button" class="button page-ai-select-none-menu">Select None</button>
                            </div>
                            <div class="page-ai-menu-list">
                                <?php foreach ($menu_items_by_menu as $menu_id => $menu_data): ?>
                                    <div class="page-ai-menu-group">
                                        <h4><?php echo esc_html($menu_data['name']); ?></h4>
                                        <?php
                                        if (!empty($menu_data['items_by_parent'])):
                                            display_menu_items_hierarchically(0, $menu_data['items_by_parent'], $excluded_menu_items);
                                        else:
                                        ?>
                                            <p>No page menu items found in this menu.</p>
                                        <?php endif; ?>
                                    </div>
                                <?php endforeach; ?>
                            </div>
                            <p class="description">
                                <?php if ($selection_mode === 'exclude'): ?>
                                    Select menu items that link to pages that should be excluded from automatic category assignment.
                                <?php else: ?>
                                    Select menu items that link to pages that should be included in automatic category assignment.
                                <?php endif; ?>
                            </p>
                        <?php else: ?>
                            <p>No menus found. Create a menu first.</p>
                        <?php endif; ?>
                    </div>
                </div>
                <?php
            }
        ],
        'only_remove_from_excluded' => [
            'title' => 'Category Removal Behavior',
            'icon' => 'dashicons-editor-removeformatting',
            'description' => 'Control how categories are removed from pages.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[only_remove_from_excluded]" <?php checked($options['only_remove_from_excluded'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Only Remove Categories from Excluded Pages</span>
                    </div>
                    <p class="description">When enabled, categories will only be removed from excluded pages and default pages. Other pages will keep their existing categories.</p>
                </div>
                <?php
            }
        ],
        'categories_per_page' => [
            'title' => 'Categories Per Page',
            'icon' => 'dashicons-tag',
            'description' => 'Set the maximum number of categories to assign to each page.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <label for="page_ai_categories_per_page">Maximum Categories</label>
                    <input type="number" id="page_ai_categories_per_page" name="page_ai_categories_options[categories_per_page]" min="1" value="<?php echo intval($options['categories_per_page']); ?>" class="small-text">
                    <p class="description">Maximum number of categories to assign to each page (no upper limit)</p>
                </div>
                <?php
            }
        ],
        'use_existing_categories' => [
            'title' => 'Existing Categories',
            'icon' => 'dashicons-category',
            'description' => 'Control whether to use existing categories or create new ones.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[use_existing_categories]" <?php checked($options['use_existing_categories'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Use Existing Categories</span>
                    </div>
                    <p class="description">When enabled, the plugin will try to use existing categories</p>
                </div>

                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[create_new_categories]" <?php checked($options['create_new_categories'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Create New Categories</span>
                    </div>
                    <p class="description">When enabled, the plugin will create new categories as needed</p>
                </div>
                <?php
            }
        ],
        'unique_categories' => [
            'title' => 'Category Uniqueness',
            'icon' => 'dashicons-filter',
            'description' => 'Control how categories are distributed across pages.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[unique_categories]" <?php checked($options['unique_categories'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Unique Categories Per Page</span>
                    </div>
                    <p class="description">Ensure each page has unique categories not used by other pages. When duplicates are found, they will be automatically replaced with new AI-generated categories.</p>
                </div>

                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[strict_unique_categories]" <?php checked($options['strict_unique_categories'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Strict Unique Categories</span>
                    </div>
                    <p class="description">Ensure each category is used by only one page. When a category is found on multiple pages, it will be kept on one page and replaced with similar categories on other pages.</p>
                </div>
                <?php
            }
        ],
        'exclude_default_pages' => [
            'title' => 'Page Exclusions',
            'icon' => 'dashicons-dismiss',
            'description' => 'Control which pages should be excluded from category assignment.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[exclude_default_pages]" <?php checked($options['exclude_default_pages'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Exclude Default WordPress Pages</span>
                    </div>
                    <p class="description">Exclude WordPress default pages (like Home and Privacy Policy) from category assignment.</p>
                </div>
                <?php
            }
        ],
        'force_reassign' => [
            'title' => 'Force Reassignment',
            'icon' => 'dashicons-update',
            'description' => 'Control whether to reassign categories to pages that already have them.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[force_reassign]" <?php checked($options['force_reassign'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Force Reassign Categories</span>
                    </div>
                    <p class="description">Force reassign categories even to pages that already have categories</p>
                </div>
                <?php
            }
        ],
        'error_logging' => [
            'title' => 'Logging & Debugging',
            'icon' => 'dashicons-code-standards',
            'description' => 'Control error logging and debugging options.',
            'content' => function() {
                $options = page_ai_get_options();
                ?>
                <div class="page-ai-form-row">
                    <div class="page-ai-toggle-label">
                        <label class="page-ai-toggle-switch">
                            <input type="checkbox" name="page_ai_categories_options[error_logging]" <?php checked($options['error_logging'], 1); ?> value="1">
                            <span class="page-ai-toggle-slider"></span>
                        </label>
                        <span>Enable Error Logging</span>
                    </div>
                    <p class="description">Enable logging of category generation results to the error log</p>
                </div>
                <?php
            }
        ],
    ];
    ?>
    <div class="wrap">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>

        <div class="page-ai-admin-container">
            <div class="page-ai-settings-grid">
                <!-- Main Settings Column -->
                <div class="page-ai-settings-main">
                    <form action="options.php" method="post">
                        <?php settings_fields('page_ai_settings'); ?>

                        <!-- Hidden field to store settings order -->
                        <input type="hidden" id="page_ai_settings_order" name="page_ai_categories_options[settings_order]" value="<?php echo esc_attr(json_encode($settings_order)); ?>">

                        <!-- Sortable Settings Cards -->
                        <div class="page-ai-sortable-settings">
                            <?php foreach ($settings_order as $setting_id): ?>
                                <?php if (isset($settings_cards[$setting_id])): ?>
                                    <div class="page-ai-setting-card" data-setting-id="<?php echo esc_attr($setting_id); ?>">
                                        <div class="page-ai-card-header">
                                            <span>
                                                <span class="page-ai-drag-handle dashicons <?php echo esc_attr($settings_cards[$setting_id]['icon']); ?>"></span>
                                                <?php echo esc_html($settings_cards[$setting_id]['title']); ?>
                                            </span>
                                            <span class="dashicons dashicons-arrow-down-alt2"></span>
                                        </div>
                                        <div class="page-ai-card-body">
                                            <p class="description"><?php echo esc_html($settings_cards[$setting_id]['description']); ?></p>
                                            <?php call_user_func($settings_cards[$setting_id]['content']); ?>
                                        </div>
                                    </div>
                                <?php endif; ?>
                            <?php endforeach; ?>
                        </div>

                        <div class="page-ai-save-button">
                            <?php submit_button('Save Settings', 'primary', 'submit', false); ?>
                            <p class="description">Saving these settings may affect which pages receive AI-generated categories. You will be asked to confirm before changes are applied.</p>
                        </div>
                    </form>
                </div>

                <!-- Sidebar Column -->
                <div class="page-ai-settings-sidebar">
                    <!-- Status Card -->
                    <div class="page-ai-card page-ai-status-card">
                        <div class="page-ai-card-header">
                            <span>
                                <span class="dashicons dashicons-chart-bar"></span>
                                Status
                            </span>
                        </div>
                        <div class="page-ai-card-body">
                            <div class="page-ai-status-item">
                                <span>Last Check:</span>
                                <strong><?php echo esc_html($last_run['time']); ?></strong>
                            </div>
                            <div class="page-ai-status-item">
                                <span>Pages Processed:</span>
                                <strong><?php echo intval($last_run['count']); ?> of <?php echo intval($last_run['total']); ?></strong>
                            </div>

                            <?php if (!empty($last_run['debug_info'])): ?>
                            <div class="page-ai-status-item">
                                <span>Total Pages:</span>
                                <strong><?php echo intval($last_run['debug_info']['total_pages']); ?></strong>
                            </div>
                            <div class="page-ai-status-item">
                                <span>Pages Without Categories:</span>
                                <strong><?php echo intval($last_run['debug_info']['pages_without_categories']); ?></strong>
                            </div>
                            <?php if (isset($last_run['debug_info']['fixed_duplicates'])): ?>
                            <div class="page-ai-status-item">
                                <span>Duplicate Categories Fixed:</span>
                                <strong><?php echo intval($last_run['debug_info']['fixed_duplicates']); ?></strong>
                            </div>
                            <?php endif; ?>
                            <?php endif; ?>

                            <div class="page-ai-action-buttons">
                                <form method="post" action="admin-post.php">
                                    <input type="hidden" name="action" value="page_ai_run_now">
                                    <?php wp_nonce_field('page_ai_run_now', 'page_ai_nonce'); ?>
                                    <button type="submit" class="button button-primary">
                                        <span class="dashicons dashicons-update"></span>
                                        Run Category Assignment Now
                                    </button>
                                </form>

                                <form method="post" action="admin-post.php">
                                    <input type="hidden" name="action" value="page_ai_clear_cache">
                                    <?php wp_nonce_field('page_ai_clear_cache', 'page_ai_cache_nonce'); ?>
                                    <button type="submit" class="button">
                                        <span class="dashicons dashicons-trash"></span>
                                        Clear Plugin Cache
                                    </button>
                                </form>
                            </div>
                        </div>
                    </div>

                    <!-- Help Card -->
                    <div class="page-ai-card">
                        <div class="page-ai-card-header">
                            <span>
                                <span class="dashicons dashicons-editor-help"></span>
                                Help & Support
                            </span>
                        </div>
                        <div class="page-ai-card-body">
                            <p>Need help with Page AI Categories? Here are some resources:</p>
                            <ul>
                                <li><a href="#" target="_blank">Documentation</a></li>
                                <li><a href="#" target="_blank">FAQ</a></li>
                                <li><a href="#" target="_blank">Support Forum</a></li>
                            </ul>
                            <p><strong>Version:</strong> <?php echo PLUGIN_VERSION; ?></p>
                        </div>
                    </div>
                </div>
            </div>

            <?php if (!empty($last_run['debug_info']) && current_user_can('manage_options')): ?>
            <!-- Debug Information (Collapsible) -->
            <div class="page-ai-card">
                <div class="page-ai-card-header">
                    <span>
                        <span class="dashicons dashicons-code-standards"></span>
                        Debug Information
                    </span>
                    <span class="dashicons dashicons-arrow-down-alt2"></span>
                </div>
                <div class="page-ai-card-body page-ai-debug-info">
                    <?php if (isset($last_run['debug_info']['menus'])): ?>
                    <h3>Menus</h3>
                    <p><strong>Total Menus:</strong> <?php echo count($last_run['debug_info']['menus']); ?></p>
                    <p><strong>Menu Pages:</strong> <?php echo intval($last_run['debug_info']['menu_pages_count']); ?></p>

                    <?php if (!empty($last_run['debug_info']['menus'])): ?>
                    <table class="widefat striped">
                        <thead>
                            <tr>
                                <th>Menu ID</th>
                                <th>Menu Name</th>
                                <th>Menu Items</th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php foreach ($last_run['debug_info']['menus'] as $menu): ?>
                            <tr>
                                <td><?php echo intval($menu->term_id); ?></td>
                                <td><?php echo esc_html($menu->name); ?></td>
                                <td><?php echo intval($menu->count); ?></td>
                            </tr>
                            <?php endforeach; ?>
                        </tbody>
                    </table>
                    <?php endif; ?>

                    <?php if (!empty($last_run['debug_info']['menu_pages'])): ?>
                    <h4>Menu Items Linking to Pages</h4>
                    <table class="widefat striped">
                        <thead>
                            <tr>
                                <th>Menu Name</th>
                                <th>Item Title</th>
                                <th>Page ID</th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php foreach ($last_run['debug_info']['menu_pages'] as $item): ?>
                            <tr>
                                <td><?php echo esc_html($item['menu_name']); ?></td>
                                <td><?php echo esc_html($item['item_title']); ?></td>
                                <td><?php echo intval($item['page_id']); ?></td>
                            </tr>
                            <?php endforeach; ?>
                        </tbody>
                    </table>
                    <?php endif; ?>
                    <?php endif; ?>

                    <?php if (!empty($last_run['debug_info']['page_details'])): ?>
                    <h4>Page Details</h4>
                    <table class="widefat striped">
                        <thead>
                            <tr>
                                <th>ID</th>
                                <th>Title</th>
                                <th>Default Page</th>
                                <th>Has Categories</th>
                                <th>Category Count</th>
                                <th>Category Names</th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php foreach ($last_run['debug_info']['page_details'] as $page): ?>
                            <tr>
                                <td><?php echo intval($page['id']); ?></td>
                                <td><?php echo esc_html($page['title']); ?></td>
                                <td><?php echo isset($page['is_default_page']) && $page['is_default_page'] ? 'Yes' : 'No'; ?></td>
                                <td><?php echo $page['has_categories'] ? 'Yes' : 'No'; ?></td>
                                <td><?php echo intval($page['category_count']); ?></td>
                                <td>
                                    <?php
                                    if (!empty($page['category_names'])) {
                                        echo esc_html(implode(', ', $page['category_names']));
                                    } else {
                                        echo 'None';
                                    }
                                    ?>
                                </td>
                            </tr>
                            <?php endforeach; ?>
                        </tbody>
                    </table>
                    <?php endif; ?>
                </div>
            </div>
            <?php endif; ?>
        </div>
    </div>
    <?php
}

// Handle manual run
add_action('admin_post_page_ai_run_now', 'page_ai_handle_manual_run');

/**
 * Handle manual run of category assignment
 */
function page_ai_handle_manual_run() {
    // Check nonce and capabilities
    if (!isset($_POST['page_ai_nonce']) || !wp_verify_nonce($_POST['page_ai_nonce'], 'page_ai_run_now') || !current_user_can('manage_options')) {
        wp_die('Security check failed');
    }

    // Run the recheck function
    page_ai_daily_recheck_callback();

    // Redirect back to settings page
    wp_redirect(add_query_arg(['page' => 'page_ai_categories', 'manual-run' => 'complete'], admin_url('admin.php')));
    exit;
}

// Add handler for clear cache action
add_action('admin_post_page_ai_clear_cache', 'page_ai_handle_clear_cache');

/**
 * Handle clearing the plugin cache
 */
function page_ai_handle_clear_cache() {
    // Check nonce and capabilities
    if (!isset($_POST['page_ai_cache_nonce']) || !wp_verify_nonce($_POST['page_ai_cache_nonce'], 'page_ai_clear_cache') || !current_user_can('manage_options')) {
        wp_die('Security check failed');
    }

    // Clear all plugin caches
    global $wpdb;
    $deleted = $wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE '%_transient_page_ai_%'");

    // Redirect back to settings page
    wp_redirect(add_query_arg(['page' => 'page_ai_categories', 'cache-cleared' => $deleted], admin_url('admin.php')));
    exit;
}

// Add notices for admin actions
add_action('admin_notices', function() {
    if (isset($_GET['page']) && $_GET['page'] === 'page_ai_categories') {
        // Notice for manual run completion
        if (isset($_GET['manual-run']) && $_GET['manual-run'] === 'complete') {
            $last_run = get_option('page_ai_last_run', ['count' => 0, 'total' => 0]);
            echo '<div class="notice notice-success is-dismissible"><p>Category assignment completed. Processed ' . intval($last_run['count']) . ' of ' . intval($last_run['total']) . ' pages.</p></div>';
        }

        // Notice for cache clearing
        if (isset($_GET['cache-cleared'])) {
            $count = intval($_GET['cache-cleared']);
            echo '<div class="notice notice-success is-dismissible"><p>Plugin cache cleared successfully. Removed ' . $count . ' cached items.</p></div>';
        }
    }
});

/**
 * Get menu items that link to pages
 */
function page_ai_get_menu_pages() {
    $menu_items = [];
    $menus = wp_get_nav_menus();

    if (empty($menus)) {
        return $menu_items;
    }

    foreach ($menus as $menu) {
        $items = wp_get_nav_menu_items($menu->term_id);

        if (!empty($items)) {
            foreach ($items as $item) {
                // Check if this menu item links to a page
                if ($item->object === 'page') {
                    $menu_items[] = [
                        'menu_id' => $menu->term_id,
                        'menu_name' => $menu->name,
                        'item_id' => $item->ID,
                        'item_title' => $item->title,
                        'page_id' => $item->object_id
                    ];
                }
            }
        }
    }

    return $menu_items;
}

// Core generation logic below
function page_ai_generate_categories_for_page($page_id, $page_title) {
    $options = page_ai_get_options();
    $max     = intval($options['categories_per_page']);
    $use_existing = (bool)$options['use_existing_categories'];
    $create_new   = (bool)$options['create_new_categories'];
    $exclude_default = (bool)$options['exclude_default_pages'];
    $excluded_pages = isset($options['excluded_pages']) ? $options['excluded_pages'] : [];
    $excluded_menu_items = isset($options['excluded_menu_items']) ? $options['excluded_menu_items'] : [];
    $only_remove_from_excluded = (bool)$options['only_remove_from_excluded'];

    // Get selection mode and included pages
    $selection_mode = isset($options['selection_mode']) ? $options['selection_mode'] : 'exclude';
    $included_pages = isset($options['included_pages']) ? $options['included_pages'] : [];

    // Determine if this page should be processed based on selection mode
    $should_process = true;

    if ($selection_mode === 'exclude') {
        // In exclude mode, process all pages except excluded ones
        $is_excluded = false;

        // Check if it's a default page
        if ($exclude_default && page_ai_is_default_page($page_id)) {
            $is_excluded = true;
        }

        // Check if it's in the excluded pages list
        if (in_array($page_id, $excluded_pages)) {
            $is_excluded = true;
        }

        // Check if it's in the excluded menu items list
        if (in_array($page_id, $excluded_menu_items)) {
            $is_excluded = true;
        }

        // If the page is excluded, don't process it
        if ($is_excluded) {
            // Remove any existing categories from this page
            wp_set_post_terms($page_id, [], 'category', false);
            return false;
        }
    } else {
        // In include mode, only process specifically included pages
        $is_included = false;

        // Check if it's in the included pages list
        if (in_array($page_id, $included_pages)) {
            $is_included = true;
        }

        // Check if it's in the included menu items list (which uses the same field as excluded_menu_items)
        if (in_array($page_id, $excluded_menu_items)) {
            $is_included = true;
        }

        // If the page is not included, don't process it
        if (!$is_included) {
            // If we're only removing categories from excluded pages, preserve existing categories
            if (!$only_remove_from_excluded) {
                // Remove any existing categories from this page
                wp_set_post_terms($page_id, [], 'category', false);
            }
            return false;
        }
    }

    // If we're only removing categories from excluded pages and this page isn't excluded,
    // preserve its existing categories
    if ($only_remove_from_excluded) {
        $existing_cats = wp_get_post_terms($page_id, 'category', ['fields' => 'ids']);
        if (!empty($existing_cats)) {
            return true;
        }
    }

    // Check for existing categories on this page
    $existing_cats = wp_get_post_terms($page_id, 'category', ['fields' => 'ids']);

    // If the page already has categories, preserve them and just check for duplicates
    if (!empty($existing_cats) && !$options['force_reassign']) {
        // Remove duplicates within the existing categories
        $unique_cats = page_ai_remove_duplicate_category_names($existing_cats);

        // Only update if we actually removed duplicates
        if (count($unique_cats) !== count($existing_cats)) {
            wp_set_post_terms($page_id, $unique_cats, 'category', false);
        }

        return true;
    }

    // existing-only
    if ($use_existing && ! $create_new) {
        return page_ai_assign_existing_categories($page_id, $page_title, $max);
    }

    // prompt
    $prompt = sprintf(
        "Create %d diverse but relevant concise category names for a page titled: %s. Each category should represent a different aspect or dimension of the page topic. Ensure variety while staying within the context of the page title.",
        $max, $page_title
    );

    // Get options for error logging
    $options = page_ai_get_options();
    $error_logging = (bool)$options['error_logging'];

    // Get API key from options
    $api_key = page_ai_get_api_key();

    // Check if API key is set
    if (empty($api_key)) {
        if ($error_logging) {
            error_log('Page AI Categories: API key is not set. Please configure it in the settings.');
        }
        return false;
    }

    try {
        // Add transient cache key based on prompt to avoid duplicate API calls
        $cache_key = 'page_ai_cat_' . md5($prompt);
        $cached_response = get_transient($cache_key);

        if ($cached_response !== false) {
            // Use cached response
            $body = $cached_response;
        } else {
            // Make API request
            $resp = wp_remote_post(DEEPSEEK_API_ENDPOINT, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $api_key,
                    'Content-Type'  => 'application/json',
                ],
                'body'    => wp_json_encode([
                    'model'    => DEEPSEEK_MODEL,
                    'messages' => [
                        ['role'=>'system','content'=>'Provide only concise, diverse category names that cover different aspects of the page topic. Each category should be distinct but still relevant to the page title. Return only the category names separated by commas. Do not include phrases like "relevant category names for" or "Select". Do not include numbers. Do not include the page title in the category names. Aim for variety in your categories while ensuring they all relate to the page subject.'],
                        ['role'=>'user','content'=> $prompt],
                    ],
                    'max_tokens'=>100,
                    'temperature'=>0.8, // Higher temperature for more diversity
                ]),
                'timeout'=>20,
            ]);

            // Enhanced error handling
            if (is_wp_error($resp)) {
                $error_message = $resp->get_error_message();
                if ($error_logging) {
                    error_log('Page AI Categories API Error: ' . $error_message);
                }

                // Add admin notice for API errors
                add_action('admin_notices', function() use ($error_message) {
                    echo '<div class="notice notice-error is-dismissible"><p>Page AI Categories API Error: ' . esc_html($error_message) . '</p></div>';
                });

                return false;
            }

            $status_code = wp_remote_retrieve_response_code($resp);
            $response_body = wp_remote_retrieve_body($resp);

            if ($status_code !== 200) {
                $error_message = sprintf('Status code %d - %s', $status_code, $response_body);

                if ($error_logging) {
                    error_log('Page AI Categories API Error: ' . $error_message);
                }

                // Add admin notice for API errors
                add_action('admin_notices', function() use ($error_message) {
                    echo '<div class="notice notice-error is-dismissible"><p>Page AI Categories API Error: ' . esc_html($error_message) . '</p></div>';
                });

                // For rate limiting errors, wait and retry once
                if ($status_code === 429) {
                    // Wait 2 seconds and try again
                    sleep(2);

                    $retry_resp = wp_remote_post(DEEPSEEK_API_ENDPOINT, [
                        'headers' => [
                            'Authorization' => 'Bearer ' . $api_key,
                            'Content-Type'  => 'application/json',
                        ],
                        'body'    => wp_json_encode([
                            'model'    => DEEPSEEK_MODEL,
                            'messages' => [
                                ['role'=>'system','content'=>'Provide only concise, diverse category names that cover different aspects of the page topic. Each category should be distinct but still relevant to the page title. Return only the category names separated by commas. Do not include phrases like "relevant category names for" or "Select". Do not include numbers. Do not include the page title in the category names. Aim for variety in your categories while ensuring they all relate to the page subject.'],
                                ['role'=>'user','content'=> $prompt],
                            ],
                            'max_tokens'=>100,
                            'temperature'=>0.8, // Higher temperature for more diversity
                        ]),
                        'timeout'=>20,
                    ]);

                    if (!is_wp_error($retry_resp) && wp_remote_retrieve_response_code($retry_resp) === 200) {
                        $response_body = wp_remote_retrieve_body($retry_resp);
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            }

            $body = json_decode($response_body, true);

            // Cache the response for 1 hour (3600 seconds)
            if (!empty($body['choices'][0]['message']['content'])) {
                set_transient($cache_key, $body, 3600);
            } else {
                if ($error_logging) {
                    error_log('Page AI Categories API Error: Empty response or invalid format');
                }
                return false;
            }
        }
    } catch (Exception $e) {
        $error_message = $e->getMessage();
        if ($error_logging) {
            error_log('Page AI Categories Exception: ' . $error_message);
        }

        // Add admin notice for exceptions
        add_action('admin_notices', function() use ($error_message) {
            echo '<div class="notice notice-error is-dismissible"><p>Page AI Categories Exception: ' . esc_html($error_message) . '</p></div>';
        });

        return false;
    }
    $names = array_filter(array_map('sanitize_text_field', explode(',', $body['choices'][0]['message']['content'])));
    // clean names
    $names = array_map(function($n) use ($page_title) {
        // Remove numbers
        $n = preg_replace('/[0-9]+/', '', trim($n));
        // Remove phrases like "relevant category names for" or "Select"
        $n = preg_replace('/^(relevant\s+category\s+(names|suggestions)\s+for\s+|Select\s+)/i', '', $n);
        // Remove phrases like "for [page title]"
        $n = preg_replace('/\s+for\s+"?' . preg_quote($page_title, '/') . '"?:?\s*-?\s*/i', '', $n);
        // Remove any direct reference to the page title
        $n = str_ireplace($page_title, '', $n);
        // Remove any remaining colons, dashes or quotes at the beginning
        $n = preg_replace('/^[\s:"\-–—]+/', '', $n);
        // Clean up extra spaces and trim again
        return preg_replace('/\s+/', ' ', trim($n));
    }, $names);
    // Remove empty or very short category names (less than 3 characters)
    $names = array_filter($names, function($name) {
        return strlen(trim($name)) >= 3;
    });
    $names = array_slice($names, 0, $max);

    $term_ids = [];
    $unique_categories = (bool)$options['unique_categories'];

    // Get excluded shop categories
    $excluded_shop_categories = isset($options['excluded_shop_categories']) ? $options['excluded_shop_categories'] : [];

    foreach ($names as $n) {
        $term = term_exists($n, 'category');

        // If the term exists
        if ($term) {
            $term_id = is_array($term) ? intval($term['term_id']) : intval($term);

            // Skip if this is an excluded shop category
            if (in_array($term_id, $excluded_shop_categories)) {
                continue;
            }

            // Check if we need to ensure unique categories
            if ($unique_categories && page_ai_is_category_used_by_other_pages($term_id, $page_id)) {
                // Skip this category as it's already used by another page
                continue;
            }

            $term_ids[] = $term_id;
        }
        // If the term doesn't exist and we can create new ones
        elseif (! $term && $create_new) {
            $new = wp_insert_term($n, 'category');
            if (! is_wp_error($new)) {
                $term_ids[] = intval($new['term_id']);
            }
        }
    }
    // fill from existing if needed
    if ($use_existing && count($term_ids) < $max) {
        $more = page_ai_find_relevant_existing_categories($page_title, $max - count($term_ids), $term_ids, $page_id);
        $term_ids = array_merge($term_ids, $more);
    }

    // Ensure no duplicate category IDs within this page
    $term_ids = array_values(array_unique($term_ids));

    // Check for duplicate category names (different IDs but same name)
    $term_ids = page_ai_remove_duplicate_category_names($term_ids);

    // Limit to maximum number of categories
    $term_ids = array_slice($term_ids, 0, $max);

    if ($term_ids) {
        // Replace existing categories (false = replace, true = append)
        wp_set_post_terms($page_id, $term_ids, 'category', false);
        return true;
    }
    return false;
}
/**
 * Remove categories with duplicate names from an array of term IDs
 *
 * @param array $term_ids Array of term IDs to check for duplicate names
 * @return array Filtered array of term IDs with no duplicate names
 */
function page_ai_remove_duplicate_category_names($term_ids) {
    if (empty($term_ids)) {
        return [];
    }

    $unique_terms = [];
    $used_names = [];

    foreach ($term_ids as $term_id) {
        $term = get_term($term_id, 'category');
        if (is_wp_error($term) || empty($term)) {
            continue;
        }

        $term_name = strtolower(trim($term->name));

        // Skip if we've already seen this name
        if (in_array($term_name, $used_names)) {
            continue;
        }

        $used_names[] = $term_name;
        $unique_terms[] = $term_id;
    }

    return $unique_terms;
}

/**
 * Get plugin options with defaults
 */
function page_ai_get_options() {
    $defaults = [
        'api_key' => '',
        'categories_per_page' => 10, // Increased default from 5 to 10
        'use_existing_categories' => true,
        'create_new_categories' => true,
        'error_logging' => false,
        'force_reassign' => false,
        'unique_categories' => true, // Ensure categories are unique to each page
        'strict_unique_categories' => true, // Ensure each category is used by only one page
        'exclude_default_pages' => true, // Exclude WordPress default pages like Home and Privacy Policy
        'excluded_shop_categories' => [], // Shop categories to exclude from automatic assignment
        'excluded_pages' => [], // Specific pages to exclude from category assignment
        'excluded_menu_items' => [], // Menu items to exclude from category assignment
        'included_pages' => [], // Specific pages to include in category assignment
        'selection_mode' => 'exclude', // 'exclude' or 'include' mode for page selection
        'only_remove_from_excluded' => false, // When true, only remove categories from excluded pages
        'settings_order' => json_encode([
            'api_key',
            'categories_per_page',
            'use_existing_categories',
            'create_new_categories',
            'unique_categories',
            'strict_unique_categories',
            'selection_mode',
            'exclude_default_pages',
            'excluded_shop_categories',
            'excluded_pages',
            'excluded_menu_items',
            'included_pages',
            'only_remove_from_excluded',
            'force_reassign',
            'error_logging'
        ]),
    ];

    $options = get_option('page_ai_categories_options', []);
    return wp_parse_args($options, $defaults);
}

/**
 * Validate plugin options
 *
 * @param array $input The input options
 * @return array The validated options
 */
function page_ai_validate_options($input) {
    $output = [];

    // Validate API key
    if (isset($input['api_key'])) {
        $output['api_key'] = sanitize_text_field($input['api_key']);

        // Test API key if it's not empty
        if (!empty($output['api_key'])) {
            $is_valid = page_ai_test_api_key($output['api_key']);
            if (!$is_valid) {
                add_settings_error(
                    'page_ai_categories_options',
                    'invalid_api_key',
                    'The DeepSeek API key appears to be invalid. Please check and try again.',
                    'error'
                );
            }
        }
    }

    // Validate categories per page
    if (isset($input['categories_per_page'])) {
        $output['categories_per_page'] = intval($input['categories_per_page']);
        if ($output['categories_per_page'] < 1) {
            $output['categories_per_page'] = 1;
        }
    }

    // Validate checkboxes
    $checkboxes = [
        'use_existing_categories',
        'create_new_categories',
        'error_logging',
        'force_reassign',
        'unique_categories',
        'strict_unique_categories',
        'exclude_default_pages',
        'only_remove_from_excluded'
    ];

    foreach ($checkboxes as $checkbox) {
        $output[$checkbox] = isset($input[$checkbox]) ? 1 : 0;
    }

    // Validate excluded shop categories
    if (isset($input['excluded_shop_categories']) && is_array($input['excluded_shop_categories'])) {
        $output['excluded_shop_categories'] = array_map('intval', $input['excluded_shop_categories']);
    } else {
        $output['excluded_shop_categories'] = [];
    }

    // Validate excluded pages
    if (isset($input['excluded_pages']) && is_array($input['excluded_pages'])) {
        $output['excluded_pages'] = array_map('intval', $input['excluded_pages']);
    } else {
        $output['excluded_pages'] = [];
    }

    // Validate excluded menu items
    if (isset($input['excluded_menu_items']) && is_array($input['excluded_menu_items'])) {
        $output['excluded_menu_items'] = array_map('intval', $input['excluded_menu_items']);
    } else {
        $output['excluded_menu_items'] = [];
    }

    // Validate included pages
    if (isset($input['included_pages']) && is_array($input['included_pages'])) {
        $output['included_pages'] = array_map('intval', $input['included_pages']);
    } else {
        $output['included_pages'] = [];
    }

    // Validate selection mode
    if (isset($input['selection_mode']) && in_array($input['selection_mode'], ['exclude', 'include'])) {
        $output['selection_mode'] = sanitize_text_field($input['selection_mode']);
    } else {
        $output['selection_mode'] = 'exclude';
    }

    // Validate settings order
    if (isset($input['settings_order'])) {
        $settings_order = json_decode($input['settings_order'], true);
        if (is_array($settings_order)) {
            // Ensure all required settings are included
            $required_settings = [
                'api_key',
                'categories_per_page',
                'use_existing_categories',
                'create_new_categories',
                'unique_categories',
                'strict_unique_categories',
                'exclude_default_pages',
                'force_reassign',
                'error_logging'
            ];

            // Add any missing settings to the end of the order
            foreach ($required_settings as $setting) {
                if (!in_array($setting, $settings_order)) {
                    $settings_order[] = $setting;
                }
            }

            $output['settings_order'] = json_encode($settings_order);
        }
    }

    return $output;
}

/**
 * Test if an API key is valid
 *
 * @param string $api_key The API key to test
 * @return bool True if valid, false otherwise
 */
function page_ai_test_api_key($api_key) {
    try {
        $resp = wp_remote_post(DEEPSEEK_API_ENDPOINT, [
            'headers' => [
                'Authorization' => 'Bearer ' . $api_key,
                'Content-Type'  => 'application/json',
            ],
            'body'    => wp_json_encode([
                'model'    => DEEPSEEK_MODEL,
                'messages' => [
                    ['role'=>'system','content'=>'This is a test message to validate the API key.'],
                    ['role'=>'user','content'=> 'Hello'],
                ],
                'max_tokens'=> 10,
            ]),
            'timeout'=> 10,
        ]);

        if (is_wp_error($resp)) {
            return false;
        }

        $status_code = wp_remote_retrieve_response_code($resp);
        return $status_code === 200;
    } catch (Exception $e) {
        return false;
    }
}

/**
 * Assign existing categories to a page based on relevance
 *
 * @param int $page_id The page ID to assign categories to
 * @param string $page_title The page title to find relevant categories for
 * @param int $max Maximum number of categories to assign
 * @return bool True if categories were assigned, false otherwise
 */
function page_ai_assign_existing_categories($page_id, $page_title, $max) {
    $existing_ids = page_ai_find_relevant_existing_categories($page_title, $max, [], $page_id);

    if (empty($existing_ids)) {
        return false;
    }

    // Ensure no duplicate category IDs
    $existing_ids = array_values(array_unique($existing_ids));

    // Check for duplicate category names (different IDs but same name)
    $existing_ids = page_ai_remove_duplicate_category_names($existing_ids);

    // Limit to maximum number of categories
    $existing_ids = array_slice($existing_ids, 0, $max);

    // Replace existing categories (false = replace, true = append)
    wp_set_post_terms($page_id, $existing_ids, 'category', false);
    return true;
}

/**
 * Find relevant existing categories based on page title
 *
 * @param string $page_title The page title to find relevant categories for
 * @param int $max Maximum number of categories to return
 * @param array $exclude_ids Category IDs to exclude
 * @param int $page_id The current page ID (optional)
 * @return array Array of category IDs
 */
function page_ai_find_relevant_existing_categories($page_title, $max, $exclude_ids = [], $page_id = 0) {
    // Get plugin options
    $options = page_ai_get_options();
    $unique_categories = (bool)$options['unique_categories'];
    $excluded_shop_categories = isset($options['excluded_shop_categories']) ? $options['excluded_shop_categories'] : [];

    // Combine excluded IDs with shop categories
    $all_excluded_ids = array_merge($exclude_ids, $excluded_shop_categories);

    // Get all categories
    $args = [
        'taxonomy' => 'category',
        'hide_empty' => false,
        'fields' => 'all',
        'exclude' => $all_excluded_ids,
    ];

    $categories = get_terms($args);
    if (is_wp_error($categories) || empty($categories)) {
        return [];
    }

    // Simple relevance scoring based on word matching
    $title_words = explode(' ', strtolower(sanitize_text_field($page_title)));
    $title_words = array_filter($title_words, function($word) {
        return strlen($word) > 3; // Skip short words
    });

    $scored_categories = [];
    foreach ($categories as $category) {
        $score = 0;
        $cat_name = strtolower($category->name);
        $cat_words = explode(' ', $cat_name);

        // Score based on exact word matches
        foreach ($title_words as $word) {
            if (strpos($cat_name, $word) !== false) {
                $score += 2;
            }
        }

        // Score based on partial matches
        foreach ($title_words as $word) {
            foreach ($cat_words as $cat_word) {
                if (strlen($word) > 3 && strlen($cat_word) > 3) {
                    similar_text($word, $cat_word, $percent);
                    if ($percent > 70) {
                        $score += 1;
                    }
                }
            }
        }

        if ($score > 0) {
            $scored_categories[$category->term_id] = $score;
        }
    }

    // Sort by score (highest first)
    arsort($scored_categories);

    // Filter out categories that are already used by other pages if unique_categories is enabled
    if ($unique_categories && $page_id > 0) {
        foreach ($scored_categories as $cat_id => $score) {
            if (page_ai_is_category_used_by_other_pages($cat_id, $page_id)) {
                unset($scored_categories[$cat_id]);
            }
        }
    }

    // Return top categories
    return array_slice(array_keys($scored_categories), 0, $max);
}

// Add "Generate Categories Now" button to the Publish box
add_action('post_submitbox_start', function() {
    global $post;
    if ($post->post_type === 'page') {
        echo '<div class="misc-pub-section">'
            . '<button type="button" class="button" id="page-ai-generate-btn">Generate Categories Now</button>'
          . '</div>';
    }
});

// Handle button click via AJAX
add_action('admin_footer', function() {
    global $post;
    if ($post->post_type === 'page') {
        ?>
        <script>
        jQuery(function($){
            $('#page-ai-generate-btn').on('click', function(){
                $.post(ajaxurl, {
                    action: 'page_ai_generate_ajax',
                    post_id: <?php echo $post->ID;?>,
                    nonce: '<?php echo wp_create_nonce('page_ai_generate');?>'
                }, function(response){
                    if (response.success) {
                        alert('Categories generated successfully');
                    } else {
                        alert('Error generating categories');
                    }
                    // refresh category checklist
                    $.post(ajaxurl, {action:'query-terms', taxonomy:'category', post_id:<?php echo $post->ID;?>}, function(html){
                        $('.categorychecklist').html(html);
                    });
                });
            });
        });
        </script>
        <?php
    }
});

// AJAX handler
add_action('wp_ajax_page_ai_generate_ajax', function(){
    check_ajax_referer('page_ai_generate','nonce');
    $post_id = intval($_POST['post_id']);
    $page = get_post($post_id);
    if ($page && $page->post_type === 'page') {
        // Check if the page already has categories
        $existing_cats = wp_get_post_terms($post_id, 'category', ['fields' => 'ids']);
        $had_categories = !empty($existing_cats);

        // Generate categories for this page
        page_ai_generate_categories_for_page($post_id, $page->post_title);

        // Check for and fix duplicate categories if the option is enabled
        $options = page_ai_get_options();
        if ((bool)$options['unique_categories']) {
            // Get all pages to check for duplicates
            $all_pages = get_posts([
                'post_type'      => 'page',
                'post_status'    => 'publish',
                'posts_per_page' => -1,
                'fields'         => 'ids',
            ]);

            // If the page had categories before, prioritize it to keep its categories
            if ($had_categories) {
                // Put this page first in the list to prioritize it
                $other_pages = array_diff($all_pages, [$post_id]);
                page_ai_fix_duplicate_categories(array_merge([$post_id], $other_pages), $post_id);
            } else {
                // Otherwise, just check for duplicates normally
                page_ai_fix_duplicate_categories($all_pages, $post_id);
            }
        }

        wp_send_json_success();
    }
    wp_send_json_error();
});

/**
 * Check if a page is a default WordPress page
 *
 * @param int $page_id The page ID to check
 * @return bool True if the page is a default WordPress page, false otherwise
 */
function page_ai_is_default_page($page_id) {
    // Get the page
    $page = get_post($page_id);
    if (!$page) {
        return false;
    }

    // Check if it's the front page or privacy policy page
    $front_page_id = get_option('page_on_front');
    $privacy_page_id = get_option('wp_page_for_privacy_policy');

    if ($page_id == $front_page_id || $page_id == $privacy_page_id) {
        return true;
    }

    // Check for common default page titles
    $default_titles = [
        'Home',
        'Privacy Policy',
        'Sample Page',
        'About',
        'Blog',
        'Contact'
    ];

    if (in_array($page->post_title, $default_titles)) {
        return true;
    }

    // Check if it's one of the first pages created (likely default)
    if ($page->post_status == 'publish' && $page->post_type == 'page' && $page->ID <= 2) {
        return true;
    }

    return false;
}

/**
 * Fix duplicate categories across pages
 *
 * @param array $page_ids Array of page IDs to check
 * @param int $priority_page_id Page ID that should keep its categories in case of conflicts
 * @return int Number of fixed duplicates
 */
function page_ai_fix_duplicate_categories($page_ids, $priority_page_id = 0) {
    if (empty($page_ids)) {
        return 0;
    }

    // Check if we should process in batches
    $total_pages = count($page_ids);
    $batch_size = 50; // Process 50 pages at a time to avoid memory issues
    $fixed_count = 0;

    // Use transient cache to avoid rebuilding the map on every request
    $cache_key = 'page_ai_category_map_' . md5(implode(',', $page_ids));
    $category_page_map = get_transient($cache_key);

    if ($category_page_map === false) {
        $category_page_map = [];

        // Process in batches if there are many pages
        if ($total_pages > $batch_size) {
            for ($i = 0; $i < $total_pages; $i += $batch_size) {
                $batch = array_slice($page_ids, $i, $batch_size);
                $fixed_count += page_ai_process_duplicate_batch($batch, $priority_page_id, $category_page_map);
            }

            // Cache the category map for 1 hour
            set_transient($cache_key, $category_page_map, 3600);

            return $fixed_count;
        }

        // For smaller sites, build the map in one go
        foreach ($page_ids as $page_id) {
            $cats = wp_get_post_terms($page_id, 'category', ['fields' => 'ids']);
            foreach ($cats as $cat_id) {
                if (!isset($category_page_map[$cat_id])) {
                    $category_page_map[$cat_id] = [];
                }
                $category_page_map[$cat_id][] = $page_id;
            }
        }

        // Cache the category map for 1 hour
        set_transient($cache_key, $category_page_map, 3600);
    }

    // Get options for logging and settings
    $options = page_ai_get_options();
    $error_logging = (bool)$options['error_logging'];
    $strict_unique = (bool)$options['strict_unique_categories'];

    // Now check for categories used by multiple pages
    foreach ($category_page_map as $cat_id => $cat_pages) {
        if (count($cat_pages) > 1) {
            // This category is used by multiple pages

            // If strict unique categories is enabled, ensure each category is used by only one page
            if ($strict_unique) {
                // Determine which page gets to keep this category
                $keeper_page_id = $priority_page_id;

                // If no priority page is specified or it's not in the list of pages using this category,
                // pick the first page in the list
                if ($keeper_page_id === 0 || !in_array($keeper_page_id, $cat_pages)) {
                    $keeper_page_id = $cat_pages[0];
                }

                if ($error_logging) {
                    $term = get_term($cat_id, 'category');
                    $term_name = $term ? $term->name : "Category #$cat_id";
                    error_log("Page AI Categories: Category '$term_name' is used by multiple pages. Page #$keeper_page_id will keep it.");
                }

                // Remove this category from all other pages and generate similar but unique categories
                foreach ($cat_pages as $page_id) {
                    // Skip the keeper page
                    if ($page_id == $keeper_page_id) {
                        continue;
                    }

                    // Replace the duplicate category on this page with a similar but unique one
                    if (page_ai_replace_duplicate_category($page_id, $cat_id)) {
                        $fixed_count++;
                    }
                }
            } else {
                // If not in strict mode, only fix duplicates for non-priority pages
                foreach ($cat_pages as $page_id) {
                    // Skip the priority page if specified
                    if ($priority_page_id > 0 && $page_id == $priority_page_id) {
                        continue;
                    }

                    // Replace the duplicate category on this page
                    if (page_ai_replace_duplicate_category($page_id, $cat_id)) {
                        $fixed_count++;
                    }
                }
            }
        }
    }

    return $fixed_count;
}

/**
 * Process a batch of pages for duplicate categories
 *
 * @param array $batch_page_ids Array of page IDs to process in this batch
 * @param int $priority_page_id Page ID that should keep its categories
 * @param array &$category_page_map Reference to the category-page map to update
 * @return int Number of fixed duplicates in this batch
 */
function page_ai_process_duplicate_batch($batch_page_ids, $priority_page_id, &$category_page_map) {
    $fixed_count = 0;

    // Build the map for this batch
    foreach ($batch_page_ids as $page_id) {
        $cats = wp_get_post_terms($page_id, 'category', ['fields' => 'ids']);
        foreach ($cats as $cat_id) {
            if (!isset($category_page_map[$cat_id])) {
                $category_page_map[$cat_id] = [];
            }
            $category_page_map[$cat_id][] = $page_id;
        }
    }

    // Get options for logging and settings
    $options = page_ai_get_options();
    $error_logging = (bool)$options['error_logging'];
    $strict_unique = (bool)$options['strict_unique_categories'];

    // Process duplicates within this batch
    foreach ($category_page_map as $cat_id => $cat_pages) {
        // Only process categories that have pages in this batch
        $batch_pages = array_intersect($cat_pages, $batch_page_ids);

        if (count($cat_pages) > 1 && !empty($batch_pages)) {
            // If strict unique categories is enabled, ensure each category is used by only one page
            if ($strict_unique) {
                // Determine which page gets to keep this category
                $keeper_page_id = $priority_page_id;

                // If no priority page is specified or it's not in the list of pages using this category,
                // pick the first page in the list
                if ($keeper_page_id === 0 || !in_array($keeper_page_id, $batch_pages)) {
                    $keeper_page_id = $batch_pages[0];
                }

                if ($error_logging) {
                    $term = get_term($cat_id, 'category');
                    $term_name = $term ? $term->name : "Category #$cat_id";
                    error_log("Page AI Categories: Category '$term_name' is used by multiple pages in batch. Page #$keeper_page_id will keep it.");
                }

                foreach ($batch_pages as $page_id) {
                    // Skip the keeper page
                    if ($page_id == $keeper_page_id) {
                        continue;
                    }

                    // Replace the duplicate category on this page with a similar but unique one
                    if (page_ai_replace_duplicate_category($page_id, $cat_id)) {
                        $fixed_count++;
                    }
                }
            } else {
                // If not in strict mode, only fix duplicates for non-priority pages
                foreach ($batch_pages as $page_id) {
                    // Skip the priority page if specified
                    if ($priority_page_id > 0 && $page_id == $priority_page_id) {
                        continue;
                    }

                    // Replace the duplicate category on this page
                    if (page_ai_replace_duplicate_category($page_id, $cat_id)) {
                        $fixed_count++;
                    }
                }
            }
        }
    }

    return $fixed_count;
}

/**
 * Check if a category is already used by other pages
 *
 * @param int $category_id The category ID to check
 * @param int $current_page_id The current page ID to exclude from the check
 * @return bool True if the category is used by other pages, false otherwise
 */
function page_ai_is_category_used_by_other_pages($category_id, $current_page_id) {
    // Use a transient cache to avoid repeated queries
    $cache_key = 'page_ai_cat_usage_' . $category_id . '_' . $current_page_id;
    $cached_result = get_transient($cache_key);

    if ($cached_result !== false) {
        return $cached_result;
    }

    // Use a direct SQL query for better performance
    global $wpdb;

    $query = $wpdb->prepare(
        "SELECT COUNT(p.ID)
        FROM {$wpdb->posts} p
        JOIN {$wpdb->term_relationships} tr ON p.ID = tr.object_id
        JOIN {$wpdb->term_taxonomy} tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
        WHERE p.post_type = 'page'
        AND p.post_status = 'publish'
        AND p.ID != %d
        AND tt.taxonomy = 'category'
        AND tt.term_id = %d
        LIMIT 1",
        $current_page_id,
        $category_id
    );

    $count = $wpdb->get_var($query);
    $result = ($count > 0);

    // Cache the result for 1 hour
    set_transient($cache_key, $result, 3600);

    return $result;
}

/**
 * Get pages that use a specific category
 *
 * @param int $category_id The category ID to check
 * @param int $exclude_page_id The page ID to exclude from the results
 * @return array Array of page IDs that use this category
 */
function page_ai_get_pages_using_category($category_id, $exclude_page_id = 0) {
    // Use a transient cache to avoid repeated queries
    $cache_key = 'page_ai_cat_pages_' . $category_id . '_' . $exclude_page_id;
    $cached_pages = get_transient($cache_key);

    if ($cached_pages !== false) {
        return $cached_pages;
    }

    // Use a direct SQL query for better performance
    global $wpdb;

    $query = $wpdb->prepare(
        "SELECT p.ID
        FROM {$wpdb->posts} p
        JOIN {$wpdb->term_relationships} tr ON p.ID = tr.object_id
        JOIN {$wpdb->term_taxonomy} tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
        WHERE p.post_type = 'page'
        AND p.post_status = 'publish'
        AND tt.taxonomy = 'category'
        AND tt.term_id = %d",
        $category_id
    );

    // Add exclusion if needed
    if ($exclude_page_id > 0) {
        $query .= $wpdb->prepare(" AND p.ID != %d", $exclude_page_id);
    }

    $pages = $wpdb->get_col($query);

    // Cache the result for 1 hour
    set_transient($cache_key, $pages, 3600);

    return $pages;
}

/**
 * Remove a category from a page and generate a new unique one
 *
 * @param int $page_id The page ID to update
 * @param int $category_id The category ID to remove
 * @return bool True if successful, false otherwise
 */
function page_ai_replace_duplicate_category($page_id, $category_id) {
    // Get current categories for this page
    $current_cats = wp_get_post_terms($page_id, 'category', ['fields' => 'ids']);

    // Remove the duplicate category
    $new_cats = array_diff($current_cats, [$category_id]);

    // Get the page title for generating a new category
    $page = get_post($page_id);
    if (!$page) {
        return false;
    }

    // Generate a new unique category
    $options = page_ai_get_options();
    $create_new = (bool)$options['create_new_categories'];

    if ($create_new) {
        // Get the category name we're replacing
        $term = get_term($category_id, 'category');
        $category_name = $term ? $term->name : '';

        // Generate a similar but unique category
        $prompt = sprintf(
            "Create 1 unique, concise, single-word or short phrase category name for a page titled: %s. Make it similar in meaning to '%s' but different enough to be distinct. The new category should be related to the page topic but not identical to '%s'.",
            $page->post_title,
            $category_name,
            $category_name
        );

        // Get API key from options
        $api_key = page_ai_get_api_key();
        $options = page_ai_get_options();
        $error_logging = (bool)$options['error_logging'];

        // Check if API key is set
        if (empty($api_key)) {
            if ($error_logging) {
                error_log('Page AI Categories: API key is not set. Please configure it in the settings.');
            }
            return false;
        }

        try {
            // Add transient cache key based on prompt to avoid duplicate API calls
            $cache_key = 'page_ai_cat_single_' . md5($prompt);
            $cached_response = get_transient($cache_key);

            if ($cached_response !== false) {
                // Use cached response
                $body = $cached_response;
            } else {
                // Make API request
                $resp = wp_remote_post(DEEPSEEK_API_ENDPOINT, [
                    'headers' => [
                        'Authorization' => 'Bearer ' . $api_key,
                        'Content-Type'  => 'application/json',
                    ],
                    'body'    => wp_json_encode([
                        'model'    => DEEPSEEK_MODEL,
                        'messages' => [
                            ['role'=>'system','content'=>'Provide only a single unique category name that is similar in meaning but distinct from the original category. The new category should be related to the page topic but not identical to the original. Return only the category name without any additional text.'],
                            ['role'=>'user','content'=> $prompt],
                        ],
                        'max_tokens'=>50,
                        'temperature'=>0.8, // Higher temperature for more uniqueness
                    ]),
                    'timeout'=>20,
                ]);

                if (is_wp_error($resp)) {
                    if ($error_logging) {
                        error_log('Page AI Categories API Error: ' . $resp->get_error_message());
                    }
                    return false;
                }

                $status_code = wp_remote_retrieve_response_code($resp);
                $response_body = wp_remote_retrieve_body($resp);

                if ($status_code !== 200) {
                    if ($error_logging) {
                        error_log(sprintf(
                            'Page AI Categories API Error: Status code %d - %s',
                            $status_code,
                            $response_body
                        ));
                    }
                    return false;
                }

                $body = json_decode($response_body, true);

                // Cache the response for 1 hour (3600 seconds)
                if (!empty($body['choices'][0]['message']['content'])) {
                    set_transient($cache_key, $body, 3600);
                }
            }

            if (!empty($body['choices'][0]['message']['content'])) {
                $new_cat_name = trim($body['choices'][0]['message']['content']);

                // Clean up the category name
                $new_cat_name = preg_replace('/[0-9]+/', '', $new_cat_name);
                $new_cat_name = preg_replace('/^(relevant\s+category\s+(names|suggestions)\s+for\s+|Select\s+)/i', '', $new_cat_name);
                $new_cat_name = preg_replace('/^[\s:"\-–—]+/', '', $new_cat_name);
                $new_cat_name = preg_replace('/\s+/', ' ', trim($new_cat_name));

                // Create the new category
                if (strlen($new_cat_name) >= 3) {
                    // Check if a category with this name already exists
                    $existing_term = term_exists($new_cat_name, 'category');

                    if ($existing_term) {
                        // Use the existing term
                        $term_id = is_array($existing_term) ? intval($existing_term['term_id']) : intval($existing_term);

                        // Check if this term is already in our new categories list
                        if (!in_array($term_id, $new_cats)) {
                            $new_cats[] = $term_id;
                        }
                    } else {
                        // Create a new term
                        $new_term = wp_insert_term($new_cat_name, 'category');
                        if (!is_wp_error($new_term)) {
                            $new_cats[] = intval($new_term['term_id']);
                        }
                    }
                }
            }
        } catch (Exception $e) {
            if ($error_logging) {
                error_log('Page AI Categories Exception: ' . $e->getMessage());
            }
            // Continue without a replacement
        }
    }

    // Ensure no duplicate category IDs
    $new_cats = array_values(array_unique($new_cats));

    // Check for duplicate category names
    $new_cats = page_ai_remove_duplicate_category_names($new_cats);

    // Update the page with the new categories
    wp_set_post_terms($page_id, $new_cats, 'category', false);

    return true;
}
